@@ -322,6 +322,18 @@ typedef struct {
 #define SSL_OPT_ALL            (SSL_OPT_COMPATENVVAR|SSL_OPT_EXPORTCERTDATA|SSL_OPT_FAKEBASICAUTH)
 typedef int OptionType;
 
+
+/*
+ * Define the SSL Protocol options
+ */
+
+#define SSL_PROTOCOL_NONE  (0)
+#define SSL_PROTOCOL_SSLV2 (1<<0)
+#define SSL_PROTOCOL_SSLV3 (1<<1)
+#define SSL_PROTOCOL_TLSV1 (1<<2)
+#define SSL_PROTOCOL_ALL   (SSL_PROTOCOL_SSLV2|SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1)
+typedef int ProtocolType;
+
 /*
  * Define the SSL verify levels
  */
@@ -463,6 +475,7 @@ typedef struct {
     int          nSessionCacheTimeout;
     int          nPassPhraseDialogType;
     char        *szPassPhraseDialogPath;
+    ProtocolType nProtocol;
 } SSLSrvConfigRec;
 
 /*
@@ -515,6 +528,7 @@ const char  *ssl_cmd_SSLSessionCache(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLSessionCacheTimeout(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLLog(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLLogLevel(cmd_parms *, char *, char *);
+const char  *ssl_cmd_SSLProtocol(cmd_parms *, char *, const char *);
 const char  *ssl_cmd_SSLOptions(cmd_parms *, SSLDirConfigRec *, const char *);
 const char  *ssl_cmd_SSLRequireSSL(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLRequire(cmd_parms *, SSLDirConfigRec *, char *);
