@@ -321,6 +321,8 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     RSA **ppRSA;
     X509 **ppX509;
     SSL_CTX *ctx;
+    STACK *skCAList;
+    char *cp;
 
     /*
      * Create the server host:port string because we need it a lot
@@ -351,9 +353,28 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     /*
      *  Create the new per-server SSL context
      */
-    ssl_log(s, SSL_LOG_TRACE,
-            "Init: (%s) Creating new SSL context", cpVHostID);
-    ctx = SSL_CTX_new(SSLv23_server_method());
+    if (sc->nProtocol == SSL_PROTOCOL_NONE) {
+        ssl_log(s, SSL_LOG_ERROR,
+                "Init: (%s) No SSL protocols available [hint: SSLProtocol]",
+                cpVHostID);
+        ssl_die();
+    }
+    cp = ap_pstrcat(p, (sc->nProtocol & SSL_PROTOCOL_SSLV2 ? "SSLv2, " : ""), 
+                       (sc->nProtocol & SSL_PROTOCOL_SSLV3 ? "SSLv3, " : ""), 
+                       (sc->nProtocol & SSL_PROTOCOL_TLSV1 ? "TLSv1, " : ""), NULL);
+    cp[strlen(cp)-2] = NUL;
+    ssl_log(s, SSL_LOG_TRACE, 
+            "Init: (%s) Creating new SSL context (protocols: %s)", cpVHostID, cp);
+    if (sc->nProtocol == SSL_PROTOCOL_SSLV2)
+        ctx = SSL_CTX_new(SSLv2_server_method());  /* only SSLv2 is left */ 
+    else
+        ctx = SSL_CTX_new(SSLv23_server_method()); /* be more flexible */
+    if (!(sc->nProtocol & SSL_PROTOCOL_SSLV2))
+        SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv2);
+    if (!(sc->nProtocol & SSL_PROTOCOL_SSLV3))
+        SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv3);
+    if (!(sc->nProtocol & SSL_PROTOCOL_TLSV1)) 
+        SSL_CTX_set_options(ctx, SSL_OP_NO_TLSv1);
     SSL_CTX_set_app_data(ctx, s);
     sc->pSSLCtx = ctx;
 
@@ -391,11 +412,11 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
      * Configure Client Authentication details
      */
     if (sc->szCACertificateFile != NULL || sc->szCACertificatePath != NULL) {
-        STACK *skCAList;
         ssl_log(s, SSL_LOG_TRACE,
                 "Init: (%s) Configuring client authentication", cpVHostID);
         if (!SSL_CTX_load_verify_locations(sc->pSSLCtx,
-                sc->szCACertificateFile, sc->szCACertificatePath)) {
+                                           sc->szCACertificateFile, 
+                                           sc->szCACertificatePath)) {
             ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
                     "Init: (%s) Unable to configure verify locations "
                     "for client authentication", cpVHostID);
@@ -411,6 +432,19 @@ void ssl_init_GetCertAndKey(server_rec *s, pool *p, SSLSrvConfigRec *sc)
         SSL_CTX_set_client_CA_list(sc->pSSLCtx, skCAList);
     }
 
+    /*
+     * Give a warning when no CAs were configured but client authentication
+     * should take place. This cannot work.
+     */
+    if (sc->nVerifyClient == SSL_CVERIFY_REQUIRE) {
+        skCAList = SSL_CTX_get_client_CA_list(sc->pSSLCtx);
+        if (sk_num(skCAList) == 0)
+            ssl_log(s, SSL_LOG_WARN, 
+                    "Init: Ops, you want to request client authentication, "
+                    "but no CAs are known for verification!? "
+                    "[Hint: SSLCACertificate*]");
+    }
+
     /*
      *  Configure server certificate
      */
